#include <VRS51L3074_SDCC.h>
#include "lowlevel.h"

/*	Hardware definitions for graphic LCD displays
	
		P0	I/O	LCD D0-7
	
		P2.2	Out	LCD E
		P2.0	Out	LCD ID
		P2.1	Out	LCD R/W_

		P2.3	Out	LCD CS1
		P2.4	Out	LCD CS2
*/

/* LCD control signals */
#define	LCD_CS1	3
#define	LCD_CS2	4
#define	LCD_E	2
#define	LCD_ID	0
#define	LCD_RW	1

/* LCD status bits */
#define BUSY	7




/* Low level functions */

void LCD_SelSide(unsigned char side)
{
	if(side)	{
		P2|=(1<<LCD_CS2);	// Rise CS2
		P2&=~(1<<LCD_CS1);	// Lower CS1
	}
	else	{
		P2|=(1<<LCD_CS1);	// Rise CS1
		P2&=~(1<<LCD_CS2);	// Lower CS2
	}
}

void LCD_CalcPage(unsigned char y,unsigned char *page,unsigned char *row)
{
	*page=(y>>3);			// page = integer part of y coord /8
	*row=y-((*page)<<3);		// row = remainder
}
	
void LCD_Write(unsigned char dat)
{
  /* HD61202U: 150ns address setup, 200ns data setup, 450ns E width */
        P0=dat;	        	// write data
	P0PINCFG=0;		// P0 = Outputs, put data in bus
	P2&=~(1<<LCD_RW);	// Lower RW (Write), the ANL used is 2 cycles, so it will add a 50ns delay @40MHz
	_asm
	  DA A              ; 100ns delay, 4 cycles for a 40Mhz processor
        _endasm;
	P2|=(1<<LCD_E);	        // Rise E (50ns more for a total of 200ns)
	_asm
	  DA A
	  DA A              ; 400ns delay, 16 cycles for a 40Mhz processor
          DA A
	  DA A
        _endasm;
	P2&=~(1<<LCD_E);	// Lower E (50ns more for a total of 450ns)
	_asm
	  DA A
	  DA A              ; 400ns delay, 16 cycles for a 40Mhz processor
	  DA A              ; RET will add more delay
	  DA A
        _endasm;
}

unsigned char LCD_Read()
{
  /* HD61202U: 150ns address setup, 450ns E width, 320ns data delay */
unsigned char dat;
	P2|=(1<<LCD_RW );	// Rise RW (Read)
	P0PINCFG=0xFF;		// P0 = Inputs, remove data from bus, the MOV used is 2 cycles, so it will add a 50ns delay @40MHz
	_asm
	  da a
  ; 100ns delay, 4 cycles for a 40Mhz processor
        _endasm;
	P2|=(1<<LCD_E);	        // Rise E (50ns more for a total of 200ns)
	_asm
	  DA A
	  DA A              ; 300ns delay, 12 cycles for a 40Mhz processor
          DA A
        _endasm;
	dat=P0;	                // read data, the MOV used is 3 cycles, so it will add a 75ns delay @40MHz
	_asm
	  NOP               ; 25ns delay, 1 cycle for a 40Mhz processor
        _endasm;
	P2&=~(1<<LCD_E);	// Lower E (50ns more for a total of 450ns)
	_asm
	  DA A
	  DA A              ; 400ns delay, 16 cycles for a 40Mhz processor
	  DA A              ; return will add more delay
	  DA A
        _endasm;
	return(dat);
}

unsigned char LCD_Status()
{
unsigned char status;
        P2&=~(1<<LCD_ID);            // Lower ID (Cmd)
	do {
		status=LCD_Read();   // read status (busy flag)
	} while(status&(1<<BUSY));   // loop while busy=1
	return(status);
}

unsigned char LCD_ReadData()
{
	LCD_Status();		// ID returns low
	P2|=(1<<LCD_ID);	// Rise ID (Data)
	return(LCD_Read());
}

void LCD_WriteCmd(unsigned char cmd)
{
	LCD_Status();		// ID returns low
	LCD_Write(cmd);
}

void LCD_WriteData(unsigned char dat)
{
	LCD_Status();
	P2|=(1<<LCD_ID);	// Rise ID (Data)
	LCD_Write(dat);
}

void LCD_init ()
{
	P0=0;
        P2&=0xE0;		// Make sure Outputs are Low
        P2PINCFG&=0xE0;        // xxx00000 P2.0,1,2,3,4 = output
	//MsDelay ( 1000 );				// wait for LCD to reset itself

	LCD_SelSide(1);
	LCD_WriteCmd (0x3F);			// Display on
	LCD_SelSide(0);
	LCD_WriteCmd (0x3F);			// Display on
}
